@@ -338,7 +338,9 @@ xgb.train <- function(params = xgb.params(), data, nrounds, evals = list(),
   )
 
   # the main loop for boosting iterations
-  for (iteration in begin_iteration:end_iteration) {
+  # FIX: Handle nrounds=0 to prevent 1:0 sequence and ensure 'iteration' is defined
+  if (nrounds == 0) iteration <- end_iteration
+  for (iteration in seq(from = begin_iteration, length.out = nrounds)) {
 
     .execute.cb.before.iter(
       callbacks,
@@ -458,11 +460,11 @@ xgb.train <- function(params = xgb.params(), data, nrounds, evals = list(),
 #' See [Survival Analysis with Accelerated Failure Time](https://xgboost.readthedocs.io/en/latest/tutorials/aft_survival_analysis.html) for details.
 #' - `"multi:softmax"`: set XGBoost to do multiclass classification using the softmax objective, you also need to set num_class(number of classes)
 #' - `"multi:softprob"`: same as softmax, but output a vector of `ndata * nclass`, which can be further reshaped to `ndata * nclass` matrix. The result contains predicted probability of each data point belonging to each class.
-#' - `"rank:ndcg"`: Use LambdaMART to perform pair-wise ranking where [Normalized Discounted Cumulative Gain (NDCG)](https://en.wikipedia.org/wiki/NDCG) is maximized. This objective supports position debiasing for click data.
-#' - `"rank:map"`: Use LambdaMART to perform pair-wise ranking where [Mean Average Precision (MAP)](https://en.wikipedia.org/wiki/Mean_average_precision#Mean_average_precision) is maximized
+#' - `"rank:ndcg"`: Use LambdaMART to perform pair-wise ranking where the normalized discounted cumulative gain (NDCG) is maximized. This objective supports position debiasing for click data.
+#' - `"rank:map"`: Use LambdaMART to perform pair-wise ranking where the mean average precision (MAP) is maximized
 #' - `"rank:pairwise"`: Use LambdaRank to perform pair-wise ranking using the `ranknet` objective.
-#' - `"reg:gamma"`: gamma regression with log-link. Output is a mean of gamma distribution. It might be useful, e.g., for modeling insurance claims severity, or for any outcome that might be [gamma-distributed](https://en.wikipedia.org/wiki/Gamma_distribution#Occurrence_and_applications).
-#' - `"reg:tweedie"`: Tweedie regression with log-link. It might be useful, e.g., for modeling total loss in insurance, or for any outcome that might be [Tweedie-distributed](https://en.wikipedia.org/wiki/Tweedie_distribution#Occurrence_and_applications).
+#' - `"reg:gamma"`: gamma regression with log-link. Output is a mean of gamma distribution. It might be useful, e.g., for modeling insurance claims severity, or for any outcome that might be gamma-distributed.
+#' - `"reg:tweedie"`: Tweedie regression with log-link. It might be useful, e.g., for modeling total loss in insurance, or for any outcome that might be tweedie-distributed.
 #' @param verbosity (default=1)
 #' Verbosity of printing messages. Valid values are 0 (silent), 1 (warning), 2 (info), 3
 #' (debug). Sometimes XGBoost tries to change configurations based on heuristics, which
@@ -509,7 +511,7 @@ xgb.train <- function(params = xgb.params(), data, nrounds, evals = list(),
 #' - `"gradient_based"`: the selection probability for each training instance is proportional to the
 #'   \bold{regularized absolute value} of gradients (more specifically, \eqn{\sqrt{g^2+\lambda h^2}}).
 #'   `"subsample"` may be set to as low as 0.1 without loss of model accuracy. Note that this
-#'   sampling method is only supported when `"tree_method"` is set to `"hist"` and the device is `"cuda"`; other tree
+#'   sampling method is only supported when `"tree_method"` is set to `"hist"`; other tree
 #'   methods only support `"uniform"` sampling.
 #' @param colsample_bytree,colsample_bylevel,colsample_bynode (for Tree Booster) (default=1)
 #' This is a family of parameters for subsampling of columns.
@@ -613,31 +615,31 @@ xgb.train <- function(params = xgb.params(), data, nrounds, evals = list(),
 #' - Evaluation metrics for validation data, a default metric will be assigned according to objective (rmse for regression, and logloss for classification, `mean average precision` for ``rank:map``, etc.)
 #' - User can add multiple evaluation metrics.
 #' - The choices are listed below:
-#'   - `"rmse"`: [root mean square error](https://en.wikipedia.org/wiki/Root_mean_square_error)
+#'   - `"rmse"`: root mean square error
 #'   - `"rmsle"`: root mean square log error: \eqn{\sqrt{\frac{1}{N}[log(pred + 1) - log(label + 1)]^2}}. Default metric of `"reg:squaredlogerror"` objective. This metric reduces errors generated by outliers in dataset.  But because `log` function is employed, `"rmsle"` might output `nan` when prediction value is less than -1.  See `"reg:squaredlogerror"` for other requirements.
-#'   - `"mae"`: [mean absolute error](https://en.wikipedia.org/wiki/Mean_absolute_error)
-#'   - `"mape"`: [mean absolute percentage error](https://en.wikipedia.org/wiki/Mean_absolute_percentage_error)
-#'   - `"mphe"`: [mean Pseudo Huber error](https://en.wikipedia.org/wiki/Huber_loss). Default metric of `"reg:pseudohubererror"` objective.
-#'   - `"logloss"`: [negative log-likelihood](https://en.wikipedia.org/wiki/Log-likelihood)
+#'   - `"mae"`: mean absolute error.
+#'   - `"mape"`: mean absolute percentage error.
+#'   - `"mphe"`: mean Pseudo Huber error. Default metric of `"reg:pseudohubererror"` objective.
+#'   - `"logloss"`: negative log-likelihood.
 #'   - `"error"`: Binary classification error rate. It is calculated as `#(wrong cases)/#(all cases)`. For the predictions, the evaluation will regard the instances with prediction value larger than 0.5 as positive instances, and the others as negative instances.
 #'   - `"error@t"`: a different than 0.5 binary classification threshold value could be specified by providing a numerical value through 't'.
 #'   - `"merror"`: Multiclass classification error rate. It is calculated as `#(wrong cases)/#(all cases)`.
 #'   - `"mlogloss"`: [Multiclass logloss](https://scikit-learn.org/stable/modules/generated/sklearn.metrics.log_loss.html).
-#'   - `"auc"`: [Receiver Operating Characteristic Area under the Curve](https://en.wikipedia.org/wiki/Receiver_operating_characteristic#Area_under_the_curve).
+#'   - `"auc"`: area under the receiver-operating characteristic curve.
 #'     Available for classification and learning-to-rank tasks.
 #'     - When used with binary classification, the objective should be `"binary:logistic"` or similar functions that work on probability.
 #'     - When used with multi-class classification, objective should be `"multi:softprob"` instead of `"multi:softmax"`, as the latter doesn't output probability.  Also the AUC is calculated by 1-vs-rest with reference class weighted by class prevalence.
 #'     - When used with LTR task, the AUC is computed by comparing pairs of documents to count correctly sorted pairs.  This corresponds to pairwise learning to rank.  The implementation has some issues with average AUC around groups and distributed workers not being well-defined.
 #'     - On a single machine the AUC calculation is exact. In a distributed environment the AUC is a weighted average over the AUC of training rows on each node - therefore, distributed AUC is an approximation sensitive to the distribution of data across workers. Use another metric in distributed environments if precision and reproducibility are important.
 #'     - When input dataset contains only negative or positive samples, the output is `NaN`.  The behavior is implementation defined, for instance, `scikit-learn` returns \eqn{0.5} instead.
-#'   - `"aucpr"`: [Area under the PR curve](https://en.wikipedia.org/wiki/Precision_and_recall).
+#'   - `"aucpr"`: area under the PR curve
 #'     Available for classification and learning-to-rank tasks.
 #'
 #'     After XGBoost 1.6, both of the requirements and restrictions for using `"aucpr"` in classification problem are similar to `"auc"`.  For ranking task, only binary relevance label \eqn{y \in [0, 1]} is supported.  Different from `"map"` (mean average precision), `"aucpr"` calculates the *interpolated* area under precision recall curve using continuous interpolation.
 #'
 #'   - `"pre"`: Precision at \eqn{k}. Supports only learning to rank task.
-#'   - `"ndcg"`: [Normalized Discounted Cumulative Gain](https://en.wikipedia.org/wiki/NDCG)
-#'   - `"map"`: [Mean Average Precision](https://en.wikipedia.org/wiki/Mean_average_precision#Mean_average_precision)
+#'   - `"ndcg"`: normalized discounted cumulative gain
+#'   - `"map"`: mean average precision
 #'
 #'     The `average precision` is defined as:
 #'
@@ -687,18 +689,6 @@ xgb.train <- function(params = xgb.params(), data, nrounds, evals = list(),
 #'
 #' - For most of the cases this parameter should not be set except for growing deep
 #'   trees. After 3.0, this parameter affects GPU algorithms as well.
-#' @param extmem_single_page (for Non-Exact Tree Methods) (default = `FALSE`)
-#' This parameter is only used for the `"hist"` tree method with `device="cuda"` and
-#' `subsample != 1.0`. Before 3.0, pages were always concatenated.
-#'
-#' Version added: 3.0.0
-#'
-#' Whether the GPU-based `"hist"` tree method should concatenate the training data into a
-#' single batch instead of fetching data on-demand when external memory is used. For GPU
-#' devices that don't support address translation services, external memory training is
-#' expensive. This parameter can be used in combination with subsampling to reduce overall
-#' memory usage without significant overhead. See [Using XGBoost External Memory Version](https://xgboost.readthedocs.io/en/latest/tutorials/external_memory.html) for
-#' more information.
 #' @param max_cat_to_onehot (for Non-Exact Tree Methods)
 #' A threshold for deciding whether XGBoost should use one-hot encoding based split for
 #' categorical data.  When number of categories is lesser than the threshold then one-hot
@@ -755,8 +745,10 @@ xgb.train <- function(params = xgb.params(), data, nrounds, evals = list(),
 #' A scalar or a list of targeted quantiles (passed as a numeric vector).
 #'
 #' Version added: 2.0.0
-#' @param aft_loss_distribution (for using AFT Survival Loss (`"survival:aft"`) and Negative Log Likelihood of AFT metric (`"aft-nloglik"`))
+#' @param aft_loss_distribution (when using AFT Survival Loss (`"survival:aft"`) and Negative Log Likelihood of AFT metric (`"aft-nloglik"`))
 #' Probability Density Function, `"normal"`, `"logistic"`, or `"extreme"`.
+#' @param aft_loss_distribution_scale (when using AFT Survival Loss (`"survival:aft"`) and Negative Log Likelihood of AFT metric (`"aft-nloglik"`))
+#' Scaling factor for the AFT distribution. Range: \eqn{(0, \infty)}.
 #' @param lambdarank_pair_method (for learning to rank (`"rank:ndcg"`, `"rank:map"`, `"rank:pairwise"`)) (default = `"topk"`)
 #' How to construct pairs for pair-wise learning.
 #' - `"mean"`: Sample `lambdarank_num_pair_per_sample` pairs for each document in the query list.
@@ -830,7 +822,6 @@ xgb.params <- function(
   disable_default_eval_metric = NULL,
   use_rmm = NULL,
   max_cached_hist_node = NULL,
-  extmem_single_page = NULL,
   max_cat_to_onehot = NULL,
   max_cat_threshold = NULL,
   sample_type = NULL,
@@ -845,6 +836,7 @@ xgb.params <- function(
   huber_slope = NULL,
   quantile_alpha = NULL,
   aft_loss_distribution = NULL,
+  aft_loss_distribution_scale = NULL,
   lambdarank_pair_method = NULL,
   lambdarank_num_pair_per_sample = NULL,
   lambdarank_normalization = NULL,
