@@ -7,11 +7,11 @@
 #define DMLC_ANY_H_
 
 // This code need c++11 to compile
-#include <typeinfo>
-#include <type_traits>
-#include <utility>
 #include <algorithm>
 #include <cstring>
+#include <type_traits>
+#include <typeinfo>
+#include <utility>
 
 #include "./base.h"
 #include "./logging.h"
@@ -30,8 +30,8 @@ class any;
  * \return The reference of content
  * \tparam T The type of the value to be fetched.
  */
-template<typename T>
-inline T& get(any& src);  // NOLINT(*)
+template <typename T>
+inline T &get(any &src);  // NOLINT(*)
 
 /*!
  * Get the const reference content stored in the any as type T.
@@ -43,8 +43,8 @@ inline T& get(any& src);  // NOLINT(*)
  * \return The reference of content
  * \tparam T The type of the value to be fetched.
  */
-template<typename T>
-inline const T& get(const any& src);
+template <typename T>
+inline const T &get(const any &src);
 
 /*!
  * The "unsafe" versions of get. It is required when where we know
@@ -56,8 +56,8 @@ inline const T& get(const any& src);
  * \return The reference of content
  * \tparam T The type of the value to be fetched.
  */
-template<typename T>
-inline const T& unsafe_get(const any& src);
+template <typename T>
+inline const T &unsafe_get(const any &src);
 
 /*!
  * The "unsafe" versions of get. It is required when where we know
@@ -69,8 +69,8 @@ inline const T& unsafe_get(const any& src);
  * \return The reference of content
  * \tparam T The type of the value to be fetched.
  */
-template<typename T>
-inline T& unsafe_get(any& src);  // NOLINT(*)
+template <typename T>
+inline T &unsafe_get(any &src);  // NOLINT(*)
 
 /*!
  * \brief An any class that is compatible to std::any in c++17.
@@ -95,19 +95,19 @@ class any {
    * \brief move constructor from another any
    * \param other The other any to be moved
    */
-  inline any(any&& other);  // NOLINT(*)
+  inline any(any &&other);  // NOLINT(*)
   /*!
    * \brief copy constructor
    * \param other The other any to be copied
    */
-  inline any(const any& other);  // NOLINT(*)
+  inline any(const any &other);  // NOLINT(*)
   /*!
    * \brief constructor from any types
    * \param other The other types to be constructed into any.
    * \tparam T The value type of other.
    */
-  template<typename T>
-  inline any(T&& other);  // NOLINT(*)
+  template <typename T>
+  inline any(T &&other);  // NOLINT(*)
   /*! \brief destructor */
   inline ~any();
   /*!
@@ -115,21 +115,21 @@ class any {
    * \param other The other any to be copy or moved.
    * \return self
    */
-  inline any& operator=(any&& other);
+  inline any &operator=(any &&other);
   /*!
    * \brief assign operator from other
    * \param other The other any to be copy or moved.
    * \return self
    */
-  inline any& operator=(const any& other);
+  inline any &operator=(const any &other);
   /*!
    * \brief assign operator from any type.
    * \param other The other any to be copy or moved.
    * \tparam T The value type of other.
    * \return self
    */
-  template<typename T>
-  inline any& operator=(T&& other);
+  template <typename T>
+  inline any &operator=(T &&other);
   /*!
    * \return whether the container is empty.
    */
@@ -142,80 +142,80 @@ class any {
    * swap current content with other
    * \param other The other data to be swapped.
    */
-  inline void swap(any& other); // NOLINT(*)
+  inline void swap(any &other);  // NOLINT(*)
   /*!
    * \return The type_info about the stored type.
    */
-  inline const std::type_info& type() const;
+  inline const std::type_info &type() const;
   /*! \brief Construct value of type T inplace */
-  template<typename T, typename... Args>
-  inline void construct(Args&&... args);
+  template <typename T, typename... Args>
+  inline void construct(Args &&...args);
 
  private:
   //! \cond Doxygen_Suppress
   // declare of helper class
-  template<typename T>
+  template <typename T>
   class TypeOnHeap;
-  template<typename T>
+  template <typename T>
   class TypeOnStack;
-  template<typename T>
+  template <typename T>
   class TypeInfo;
   // size of stack space, it takes 32 bytes for one any type.
-  static const size_t kStack = sizeof(void*) * 3;
-  static const size_t kAlign = sizeof(void*);
+  static const size_t kStack = sizeof(void *) * 3;
+  static const size_t kAlign = sizeof(void *);
   // container use dynamic storage only when space runs lager
   union Data {
     // stack space
     std::aligned_storage<kStack, kAlign>::type stack;
     // pointer to heap space
-    void* pheap;
+    void *pheap;
   };
   // type specific information
   struct Type {
     // destructor function
-    void (*destroy)(Data* data);
+    void (*destroy)(Data *data);
     // copy constructor
-    void (*create_from_data)(Data* dst, const Data& src);
+    void (*create_from_data)(Data *dst, const Data &src);
     // the type info function
-    const std::type_info* ptype_info;
+    const std::type_info *ptype_info;
   };
   // constant to check if data can be stored on heap.
-  template<typename T>
+  template <typename T>
   struct data_on_stack {
     static const bool value = alignof(T) <= kAlign && sizeof(T) <= kStack;
   };
   // declare friend with
-  template<typename T>
-  friend T& get(any& src);  // NOLINT(*)
-  template<typename T>
-  friend const T& get(const any& src);
-  template<typename T>
-  friend T& unsafe_get(any& src);  // NOLINT(*)
-  template<typename T>
-  friend const T& unsafe_get(const any& src);
+  template <typename T>
+  friend T &get(any &src);  // NOLINT(*)
+  template <typename T>
+  friend const T &get(const any &src);
+  template <typename T>
+  friend T &unsafe_get(any &src);  // NOLINT(*)
+  template <typename T>
+  friend const T &unsafe_get(const any &src);
   // internal construct function
-  inline void construct(any&& other);
+  inline void construct(any &&other);
   // internal construct function
-  inline void construct(const any& other);
+  inline void construct(const any &other);
   // internal function to check if type is correct.
-  template<typename T>
+  template <typename T>
   inline void check_type() const;
-  template<typename T>
+  template <typename T>
   inline void check_type_by_name() const;
   // internal type specific information
-  const Type* type_{nullptr};
+  const Type *type_{nullptr};
   // internal data
   Data data_;
 };
 
-template<typename T>
-inline any::any(T&& other) {
+template <typename T>
+inline any::any(T &&other) {
   typedef typename std::decay<T>::type DT;
   if (std::is_same<DT, any>::value) {
     this->construct(std::forward<T>(other));
   } else {
     static_assert(std::is_copy_constructible<DT>::value,
-                  "Any can only hold value that is copy constructable");
+        "Any can only hold value that is copy constructable");
     type_ = TypeInfo<DT>::get_type();
     if (data_on_stack<DT>::value) {
 
@@ -230,29 +230,29 @@ inline any::any(T&& other) {
   }
 }
 
-inline any::any(any&& other) {
+inline any::any(any &&other) {
   this->construct(std::move(other));
 }
 
-inline any::any(const any& other) {
+inline any::any(const any &other) {
   this->construct(other);
 }
 
-inline void any::construct(any&& other) {
+inline void any::construct(any &&other) {
   type_ = other.type_;
   data_ = other.data_;
   other.type_ = nullptr;
 }
 
-inline void any::construct(const any& other) {
+inline void any::construct(const any &other) {
   type_ = other.type_;
   if (type_ != nullptr) {
     type_->create_from_data(&data_, other.data_);
   }
 }
 
-template<typename T, typename... Args>
-inline void any::construct(Args&&... args) {
+template <typename T, typename... Args>
+inline void any::construct(Args &&...args) {
   clear();
   typedef typename std::decay<T>::type DT;
   type_ = TypeInfo<DT>::get_type();
@@ -272,23 +272,23 @@ inline any::~any() {
   this->clear();
 }
 
-inline any& any::operator=(any&& other) {
+inline any &any::operator=(any &&other) {
   any(std::move(other)).swap(*this);
   return *this;
 }
 
-inline any& any::operator=(const any& other) {
+inline any &any::operator=(const any &other) {
   any(other).swap(*this);
   return *this;
 }
 
-template<typename T>
-inline any& any::operator=(T&& other) {
+template <typename T>
+inline any &any::operator=(T &&other) {
   any(std::forward<T>(other)).swap(*this);
   return *this;
 }
 
-inline void any::swap(any& other) { // NOLINT(*)
+inline void any::swap(any &other) {  // NOLINT(*)
   std::swap(type_, other.type_);
   std::swap(data_, other.data_);
 }
@@ -306,7 +306,7 @@ inline bool any::empty() const {
   return type_ == nullptr;
 }
 
-inline const std::type_info& any::type() const {
+inline const std::type_info &any::type() const {
   if (type_ != nullptr) {
     return *(type_->ptype_info);
   } else {
@@ -314,94 +314,89 @@ inline const std::type_info& any::type() const {
   }
 }
 
-template<typename T>
+template <typename T>
 inline void any::check_type() const {
-  CHECK(type_ != nullptr)
-      << "The any container is empty"
-      << " requested=" << typeid(T).name();
+  CHECK(type_ != nullptr) << "The any container is empty"
+                          << " requested=" << typeid(T).name();
   CHECK(*(type_->ptype_info) == typeid(T))
       << "The stored type mismatch"
-      << " stored=" << type_->ptype_info->name()
-      << " requested=" << typeid(T).name();
+      << " stored=" << type_->ptype_info->name() << " requested=" << typeid(T).name();
 }
 
-template<typename T>
+template <typename T>
 inline void any::check_type_by_name() const {
-  CHECK(type_ != nullptr)
-      << "The any container is empty"
-      << " requested=" << typeid(T).name();
+  CHECK(type_ != nullptr) << "The any container is empty"
+                          << " requested=" << typeid(T).name();
   CHECK(strcmp(type_->ptype_info->name(), typeid(T).name()) == 0)
       << "The stored type name mismatch"
-      << " stored=" << type_->ptype_info->name()
-      << " requested=" << typeid(T).name();
+      << " stored=" << type_->ptype_info->name() << " requested=" << typeid(T).name();
 }
 
-template<typename T>
-inline const T& get(const any& src) {
+template <typename T>
+inline const T &get(const any &src) {
   src.check_type<T>();
   return *any::TypeInfo<T>::get_ptr(&(src.data_));
 }
 
-template<typename T>
-inline T& get(any& src) { // NOLINT(*)
+template <typename T>
+inline T &get(any &src) {  // NOLINT(*)
   src.check_type<T>();
   return *any::TypeInfo<T>::get_ptr(&(src.data_));
 }
 
-template<typename T>
-inline const T& unsafe_get(const any& src) {
+template <typename T>
+inline const T &unsafe_get(const any &src) {
   src.check_type_by_name<T>();
   return *any::TypeInfo<T>::get_ptr(&(src.data_));
 }
 
-template<typename T>
-inline T& unsafe_get(any& src) { // NOLINT(*)
+template <typename T>
+inline T &unsafe_get(any &src) {  // NOLINT(*)
   src.check_type_by_name<T>();
   return *any::TypeInfo<T>::get_ptr(&(src.data_));
 }
 
-template<typename T>
+template <typename T>
 class any::TypeOnHeap {
  public:
-  inline static T* get_ptr(any::Data* data) {
-    return static_cast<T*>(data->pheap);
+  inline static T *get_ptr(any::Data *data) {
+    return static_cast<T *>(data->pheap);
   }
-  inline static const T* get_ptr(const any::Data* data) {
-    return static_cast<const T*>(data->pheap);
+  inline static const T *get_ptr(const any::Data *data) {
+    return static_cast<const T *>(data->pheap);
   }
-  inline static void create_from_data(any::Data* dst, const any::Data& data) {
+  inline static void create_from_data(any::Data *dst, const any::Data &data) {
     dst->pheap = new T(*get_ptr(&data));
   }
-  inline static void destroy(Data* data) {
-    delete static_cast<T*>(data->pheap);
+  inline static void destroy(Data *data) {
+    delete static_cast<T *>(data->pheap);
   }
 };
 
-template<typename T>
+template <typename T>
 class any::TypeOnStack {
  public:
-  inline static T* get_ptr(any::Data* data) {
-    return reinterpret_cast<T*>(&(data->stack));
+  inline static T *get_ptr(any::Data *data) {
+    return reinterpret_cast<T *>(&(data->stack));
   }
-  inline static const T* get_ptr(const any::Data* data) {
-    return reinterpret_cast<const T*>(&(data->stack));
+  inline static const T *get_ptr(const any::Data *data) {
+    return reinterpret_cast<const T *>(&(data->stack));
   }
-  inline static void create_from_data(any::Data* dst, const any::Data& data) {
+  inline static void create_from_data(any::Data *dst, const any::Data &data) {
     new (&(dst->stack)) T(*get_ptr(&data));
   }
-  inline static void destroy(Data* data) {
-    T* dptr = reinterpret_cast<T*>(&(data->stack));
+  inline static void destroy(Data *data) {
+    T *dptr = reinterpret_cast<T *>(&(data->stack));
     dptr->~T();
   }
 };
 
-template<typename T>
-class any::TypeInfo
-    : public std::conditional<any::data_on_stack<T>::value,
-                              any::TypeOnStack<T>,
-                              any::TypeOnHeap<T> >::type {
+template <typename T>
+class any::TypeInfo :
+    public std::conditional<any::data_on_stack<T>::value, any::TypeOnStack<T>,
+        any::TypeOnHeap<T>>::type {
  public:
-  inline static const Type* get_type() {
+  inline static const Type *get_type() {
     static TypeInfo<T> tp;
     return &(tp.type_);
   }
