@@ -1,5 +1,5 @@
 /**
- * Copyright 2015-2025, XGBoost Contributors
+ * Copyright 2015-2026, XGBoost Contributors
  * \file base.h
  * \brief Defines configuration macros and basic types for xgboost.
  */
@@ -33,13 +33,6 @@
 #define XGBOOST_LOG_WITH_TIME 1
 #endif  // XGBOOST_LOG_WITH_TIME
 
-/*!
- * \brief Whether to customize global PRNG.
- */
-#ifndef XGBOOST_CUSTOMIZE_GLOBAL_PRNG
-#define XGBOOST_CUSTOMIZE_GLOBAL_PRNG 0
-#endif  // XGBOOST_CUSTOMIZE_GLOBAL_PRNG
-
 /*!
  * \brief Check if alignas(*) keyword is supported. (g++ 4.8 or higher)
  */
@@ -50,7 +43,7 @@
 #endif  // defined(__GNUC__) && ((__GNUC__ == 4 && __GNUC_MINOR__ >= 8) || __GNUC__ > 4)
 
 #if defined(__GNUC__)
-#define XGBOOST_EXPECT(cond, ret)  __builtin_expect((cond), (ret))
+#define XGBOOST_EXPECT(cond, ret) __builtin_expect((cond), (ret))
 #else
 #define XGBOOST_EXPECT(cond, ret) (cond)
 #endif  // defined(__GNUC__)
@@ -58,7 +51,7 @@
 /*!
  * \brief Tag function as usable by device
  */
-#if defined (__CUDA__) || defined(__NVCC__)
+#if defined(__CUDA__) || defined(__NVCC__)
 #define XGBOOST_DEVICE __host__ __device__
 #else
 #define XGBOOST_DEVICE
@@ -72,7 +65,6 @@
 #define XGBOOST_DEV_INLINE
 #endif  // defined(__CUDA__) || defined(__CUDACC__)
 
-
 // restrict
 #if defined(_MSC_VER)
 #define XGBOOST_RESTRICT __restrict
@@ -85,8 +77,8 @@
 /* default logic for software pre-fetching */
 #if (defined(_MSC_VER) && (defined(_M_IX86) || defined(_M_AMD64))) || defined(__INTEL_COMPILER)
 // Enable _mm_prefetch for Intel compiler and MSVC+x86
-  #define XGBOOST_MM_PREFETCH_PRESENT
-  #define XGBOOST_BUILTIN_PREFETCH_PRESENT
+#define XGBOOST_MM_PREFETCH_PRESENT
+#define XGBOOST_BUILTIN_PREFETCH_PRESENT
 #elif defined(__GNUC__)
 // Enable __builtin_prefetch for GCC
 #define XGBOOST_BUILTIN_PREFETCH_PRESENT
@@ -114,9 +106,9 @@ using bst_bin_t = std::int32_t;  // NOLINT
  */
 using bst_idx_t = std::uint64_t;  // NOLINT
 /**
- * \brief Type for tree node index.
+ * \brief Type for tree node index and tree depth.
  */
-using bst_node_t = std::int32_t;      // NOLINT
+using bst_node_t = std::int32_t;  // NOLINT
 /**
  * @brief Type for ranking group index.
  */
@@ -155,15 +147,11 @@ class GradientPairInternal {
  public:
   using ValueT = T;
 
-  inline void Add(const ValueT& grad, const ValueT& hess) {
+  inline void Add(const ValueT &grad, const ValueT &hess) {
     grad_ += grad;
     hess_ += hess;
   }
 
-  inline static void Reduce(GradientPairInternal<T>& a, const GradientPairInternal<T>& b) { // NOLINT(*)
-    a += b;
-  }
-
   GradientPairInternal() = default;
 
   XGBOOST_DEVICE GradientPairInternal(T grad, T hess) {
@@ -188,30 +176,26 @@ class GradientPairInternal {
   XGBOOST_DEVICE T GetGrad() const { return grad_; }
   XGBOOST_DEVICE T GetHess() const { return hess_; }
 
-  XGBOOST_DEVICE GradientPairInternal<T> &operator+=(
-      const GradientPairInternal<T> &rhs) {
+  XGBOOST_DEVICE GradientPairInternal<T> &operator+=(const GradientPairInternal<T> &rhs) {
     grad_ += rhs.grad_;
     hess_ += rhs.hess_;
     return *this;
   }
 
-  XGBOOST_DEVICE GradientPairInternal<T> operator+(
-      const GradientPairInternal<T> &rhs) const {
+  XGBOOST_DEVICE GradientPairInternal<T> operator+(const GradientPairInternal<T> &rhs) const {
     GradientPairInternal<T> g;
     g.grad_ = grad_ + rhs.grad_;
     g.hess_ = hess_ + rhs.hess_;
     return g;
   }
 
-  XGBOOST_DEVICE GradientPairInternal<T> &operator-=(
-      const GradientPairInternal<T> &rhs) {
+  XGBOOST_DEVICE GradientPairInternal<T> &operator-=(const GradientPairInternal<T> &rhs) {
     grad_ -= rhs.grad_;
     hess_ -= rhs.hess_;
     return *this;
   }
 
-  XGBOOST_DEVICE GradientPairInternal<T> operator-(
-      const GradientPairInternal<T> &rhs) const {
+  XGBOOST_DEVICE GradientPairInternal<T> operator-(const GradientPairInternal<T> &rhs) const {
     GradientPairInternal<T> g;
     g.grad_ = grad_ - rhs.grad_;
     g.hess_ = hess_ - rhs.hess_;
@@ -249,12 +233,10 @@ class GradientPairInternal {
   }
 
   XGBOOST_DEVICE explicit GradientPairInternal(int value) {
-    *this = GradientPairInternal<T>(static_cast<float>(value),
-                                    static_cast<float>(value));
+    *this = GradientPairInternal<T>(static_cast<float>(value), static_cast<float>(value));
   }
 
-  friend std::ostream &operator<<(std::ostream &os,
-                                  const GradientPairInternal<T> &g) {
+  friend std::ostream &operator<<(std::ostream &os, const GradientPairInternal<T> &g) {
     os << g.GetGrad() << "/" << g.GetHess();
     return os;
   }
@@ -323,8 +305,8 @@ class GradientPairInt64 {
 
 using Args = std::vector<std::pair<std::string, std::string> >;
 
-/*! \brief small eps gap for minimum split decision. */
-constexpr bst_float kRtEps = 1e-6f;
+/** @brief small eps gap for minimum split decision. */
+constexpr inline float kRtEps = 1e-6f;
 
 /*! \brief define unsigned long for openmp loop */
 using omp_ulong = dmlc::omp_ulong;  // NOLINT
