@@ -1,5 +1,5 @@
-/*!
- * Copyright 2018-2020 by Contributors
+/**
+ * Copyright 2018-2023 by Contributors
  * \file split_evaluator.h
  * \brief Used for implementing a loss term specific to decision trees. Useful for custom regularisation.
  * \author Henry Gouk
@@ -10,35 +10,32 @@
 
 #include <dmlc/registry.h>
 #include <xgboost/base.h>
-#include <utility>
-#include <vector>
-#include <limits>
+
 #include <algorithm>
+#include <limits>
+#include <vector>
 
-#include "xgboost/tree_model.h"
-#include "xgboost/host_device_vector.h"
-#include "xgboost/generic_parameters.h"
-#include "../common/transform.h"
 #include "../common/math.h"
+#include "../common/transform.h"
 #include "param.h"
+#include "xgboost/context.h"
+#include "xgboost/host_device_vector.h"
 
-namespace xgboost {
-namespace tree {
+namespace xgboost::tree {
 class TreeEvaluator {
   // hist and exact use parent id to calculate constraints.
-  static constexpr bst_node_t kRootParentId =
-      (-1 & static_cast<bst_node_t>((1U << 31) - 1));
+  static constexpr bst_node_t kRootParentId = (-1 & static_cast<bst_node_t>((1U << 31) - 1));
 
   HostDeviceVector<float> lower_bounds_;
   HostDeviceVector<float> upper_bounds_;
   HostDeviceVector<int32_t> monotone_;
-  int32_t device_;
+  DeviceOrd device_;
   bool has_constraint_;
 
  public:
-  TreeEvaluator(TrainParam const& p, bst_feature_t n_features, int32_t device) {
+  TreeEvaluator(TrainParam const& p, bst_feature_t n_features, DeviceOrd device) {
     device_ = device;
-    if (device != GenericParameter::kCpuId) {
+    if (device.IsCUDA()) {
       lower_bounds_.SetDevice(device);
       upper_bounds_.SetDevice(device);
       monotone_.SetDevice(device);
@@ -58,7 +55,7 @@ class TreeEvaluator {
       has_constraint_ = true;
     }
 
-    if (device_ != GenericParameter::kCpuId) {
+    if (device_.IsCUDA()) {
       // Pull to device early.
       lower_bounds_.ConstDeviceSpan();
       upper_bounds_.ConstDeviceSpan();
@@ -85,6 +82,7 @@ class TreeEvaluator {
                     this->CalcGainGivenWeight(param, right, wright);
 
       if (constraint == 0) {
+        // no constraint
         return gain;
       } else if (constraint > 0) {
         return wleft <= wright ? gain : negative_infinity;
@@ -100,7 +98,7 @@ class TreeEvaluator {
       if (!has_constraint) {
         return w;
       }
-
+      // Calculate bound weight
       if (nodeid == kRootParentId) {
         return w;
       } else if (w < lower[nodeid]) {
@@ -121,7 +119,7 @@ class TreeEvaluator {
     }
 
     // Fast floating point division instruction on device
-    XGBOOST_DEVICE float Divide(float a, float b) const {
+    [[nodiscard]] XGBOOST_DEVICE float Divide(float a, float b) const {
 #ifdef __CUDA_ARCH__
       return __fdividef(a, b);
 #else
@@ -153,7 +151,7 @@ class TreeEvaluator {
  public:
   /* Get a view to the evaluator that can be passed down to device. */
   template <typename ParamT = TrainParam> auto GetEvaluator() const {
-    if (device_ != GenericParameter::kCpuId) {
+    if (device_.IsCUDA()) {
       auto constraints = monotone_.ConstDevicePointer();
       return SplitEvaluator<ParamT>{constraints, lower_bounds_.ConstDevicePointer(),
                                     upper_bounds_.ConstDevicePointer(), has_constraint_};
@@ -205,16 +203,6 @@ class TreeEvaluator {
         .Eval(&lower_bounds_, &upper_bounds_, &monotone_);
   }
 };
-
-enum SplitType {
-  // numerical split
-  kNum = 0,
-  // onehot encoding based categorical split
-  kOneHot = 1,
-  // partition-based categorical split
-  kPart = 2
-};
-}  // namespace tree
-}  // namespace xgboost
+}  // namespace xgboost::tree
 
 #endif  // XGBOOST_TREE_SPLIT_EVALUATOR_H_
