@@ -1,73 +1,54 @@
-/*!
- * Copyright 2014-2022 by XGBoost Contributors
+/**
+ * Copyright 2014-2025, XGBoost Contributors
  * \file updater_prune.cc
  * \brief prune a tree given the statistics
  * \author Tianqi Chen
  */
-#include <rabit/rabit.h>
 #include <xgboost/tree_updater.h>
 
-#include <string>
 #include <memory>
 
-#include "xgboost/base.h"
-#include "xgboost/json.h"
-#include "./param.h"
-#include "../common/io.h"
 #include "../common/timer.h"
-namespace xgboost {
-namespace tree {
+#include "./param.h"
+#include "xgboost/base.h"
+#include "xgboost/gradient.h"  // for GradientContainer
+#include "xgboost/json.h"
 
+namespace xgboost::tree {
 DMLC_REGISTRY_FILE_TAG(updater_prune);
 
 /*! \brief pruner that prunes a tree after growing finishes */
-class TreePruner: public TreeUpdater {
+class TreePruner : public TreeUpdater {
  public:
-  explicit TreePruner(ObjInfo task) {
+  explicit TreePruner(Context const* ctx, ObjInfo const* task) : TreeUpdater(ctx) {
     syncher_.reset(TreeUpdater::Create("sync", ctx_, task));
     pruner_monitor_.Init("TreePruner");
   }
-  char const* Name() const override {
-    return "prune";
-  }
-
+  [[nodiscard]] char const* Name() const override { return "prune"; }
   // set training parameter
-  void Configure(const Args& args) override {
-    param_.UpdateAllowUnknown(args);
-    syncher_->Configure(args);
-  }
+  void Configure(const Args& args) override { syncher_->Configure(args); }
 
-  void LoadConfig(Json const& in) override {
-    auto const& config = get<Object const>(in);
-    FromJson(config.at("train_param"), &this->param_);
-  }
-  void SaveConfig(Json* p_out) const override {
-    auto& out = *p_out;
-    out["train_param"] = ToJson(param_);
-  }
-  bool CanModifyTree() const override {
-    return true;
-  }
+  void LoadConfig(Json const&) override {}
+  void SaveConfig(Json*) const override {}
+  [[nodiscard]] bool CanModifyTree() const override { return true; }
 
   // update the tree, do pruning
-  void Update(HostDeviceVector<GradientPair> *gpair,
-              DMatrix *p_fmat,
-              const std::vector<RegTree*> &trees) override {
+  void Update(TrainParam const* param, GradientContainer* in_gpair, DMatrix* p_fmat,
+              common::Span<HostDeviceVector<bst_node_t>> out_position,
+              const std::vector<RegTree*>& trees) override {
     pruner_monitor_.Start("PrunerUpdate");
-    // rescale learning rate according to size of trees
-    float lr = param_.learning_rate;
-    param_.learning_rate = lr / trees.size();
     for (auto tree : trees) {
-      this->DoPrune(tree);
+      this->DoPrune(param, tree);
     }
-    param_.learning_rate = lr;
-    syncher_->Update(gpair, p_fmat, trees);
+    syncher_->Update(param, in_gpair, p_fmat, out_position, trees);
     pruner_monitor_.Stop("PrunerUpdate");
   }
 
  private:
   // try to prune off current leaf
-  bst_node_t TryPruneLeaf(RegTree &tree, int nid, int depth, int npruned) { // NOLINT(*)
+  bst_node_t TryPruneLeaf(TrainParam const* param, RegTree* p_tree, int nid, int depth,
+                          int npruned) {
+    auto& tree = *p_tree;
     CHECK(tree[nid].IsLeaf());
     if (tree[nid].IsRoot()) {
       return npruned;
@@ -80,22 +61,22 @@ class TreePruner: public TreeUpdater {
     auto right = tree[pid].RightChild();
     bool balanced = tree[left].IsLeaf() &&
                     right != RegTree::kInvalidNodeId && tree[right].IsLeaf();
-    if (balanced && param_.NeedPrune(s.loss_chg, depth)) {
+    if (balanced && param->NeedPrune(s.loss_chg, depth)) {
       // need to be pruned
-      tree.ChangeToLeaf(pid, param_.learning_rate * s.base_weight);
+      tree.ChangeToLeaf(pid, param->learning_rate * s.base_weight);
       // tail recursion
-      return this->TryPruneLeaf(tree, pid, depth - 1, npruned + 2);
+      return this->TryPruneLeaf(param, p_tree, pid, depth - 1, npruned + 2);
     } else {
       return npruned;
     }
   }
   /*! \brief do pruning of a tree */
-  void DoPrune(RegTree* p_tree) {
+  void DoPrune(TrainParam const* param, RegTree* p_tree) {
     auto& tree = *p_tree;
     bst_node_t npruned = 0;
-    for (int nid = 0; nid < tree.param.num_nodes; ++nid) {
+    for (int nid = 0; nid < tree.NumNodes(); ++nid) {
       if (tree[nid].IsLeaf() && !tree[nid].IsDeleted()) {
-        npruned = this->TryPruneLeaf(tree, nid, tree.GetDepth(nid), npruned);
+        npruned = this->TryPruneLeaf(param, p_tree, nid, tree.GetDepth(nid), npruned);
       }
     }
     LOG(INFO) << "tree pruning end, "
@@ -106,15 +87,12 @@ class TreePruner: public TreeUpdater {
  private:
   // synchronizer
   std::unique_ptr<TreeUpdater> syncher_;
-  // training parameter
-  TrainParam param_;
   common::Monitor pruner_monitor_;
 };
 
 XGBOOST_REGISTER_TREE_UPDATER(TreePruner, "prune")
-.describe("Pruner that prune the tree according to statistics.")
-.set_body([](ObjInfo task) {
-    return new TreePruner(task);
-  });
-}  // namespace tree
-}  // namespace xgboost
+    .describe("Pruner that prune the tree according to statistics.")
+    .set_body([](Context const* ctx, ObjInfo const* task) {
+      return new TreePruner{ctx, task};
+    });
+}  // namespace xgboost::tree
